if select(6, GetAddOnInfo("PitBull4_" .. (debugstack():match("[o%.][d%.][u%.]les\\(.-)\\") or ""))) ~= "MISSING" then return end

local PitBull4 = _G.PitBull4
if not PitBull4 then
	error("PitBull4_Anubarak requires PitBull4")
end

--[===[@debug@
LibStub("AceLocale-3.0"):NewLocale("PitBull4_Anubarak", "enUS", true, true)
--@end-debug@]===]
local L = LibStub("AceLocale-3.0"):GetLocale("PitBull4_Anubarak")

local PitBull4_Anubarak = PitBull4:NewModule("Anubarak", "AceEvent-3.0")

PitBull4_Anubarak:SetModuleType("indicator")
PitBull4_Anubarak:SetName(L["Anub'arak"])
PitBull4_Anubarak:SetDescription(L["Display an indicator to show your Penetrating Cold target(s)."])
PitBull4_Anubarak:SetDefaults({}, {
	sort_method = "group_name_asc",
	assignments = {
		[10] = {},
		[25] = {},
	},
	announce = false,
})

function PitBull4_Anubarak:OnEnable()
	self:PARTY_MEMBERS_CHANGED()

	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "PARTY_MEMBERS_CHANGED")

	self:UpdateAll()
end

local _,_,texture = GetSpellInfo(68510)

-- spell ids of the debuff as the key and the number of targets as the value.
local penetrating_cold = {
	[68509] = 2,
	[68510] = 5,
	[66013] = 2,
	[67700] = 5,
--	[53251] = 6, -- Wild growth for testing
}

-- variables to keep track of the debuff state
local queued_pc_targets = {}
local queued_count = 0
local my_pc_targets = {}
local last_pc_applied
local raid_size

-- cache of the members of the raid that we keep sorted
-- and the pool for unneeded tables
local group_cache = {}
local pool = {}

local function new()
	local t = next(pool)
	if t then
		pool[t] = nil
	else
		t = {}
	end
	return t
end

local function delete(t)
	if t then
		pool[t] = true
	end
	return nil
end

local timer_frame = CreateFrame("Frame")
timer_frame:Hide()

-- iterate through the group cache looking for our assignments
-- turning on the indicator if needed and optionally announcing
-- them to the raid.
local function calculate_my_pc_targets()
	timer_frame:Hide()
	wipe(my_pc_targets)
	local pc_position = 1
	local db = PitBull4_Anubarak.db.profile.global
	local assignments = db.assignments[raid_size]
	local announce = db.announce
	local output_list

	for i=1,#group_cache do
		local entry = group_cache[i]
		if entry then
			local guid = entry[2]
			if queued_pc_targets[guid] then
				if assignments[pc_position] then
					--print('found',pc_position)
					my_pc_targets[guid] = true
					PitBull4_Anubarak:UpdateForGUID(guid)
				end
				if announce then
					if not output_list then
						output_list = pc_position .. "=" .. entry[3]
					else
						output_list = output_list .. "," .. pc_position .. "=" .. entry[3]
					end
				end
				pc_position = pc_position + 1
			end
		end
	end
	--print(output_list)
	if announce and output_list then
		if UnitInRaid("player") then
			if IsRaidLeader() or IsRaidOfficer() then
				SendChatMessage(output_list,"RAID_WARNING")	
			else
				SendChatMessage(output_list,"RAID")
			end
		elseif UnitInParty("player") then
			SendChatMessage(output_list,"RAID_WARNING")
		end
	end
	queued_count = 0
	wipe(queued_pc_targets)
end

-- The timer frame exists to capture the debuff if for
-- some reason the maximum number of them didn't go out.
-- This can happen if someone resisted, died just after
-- the cast succeeded but before the debuff event was
-- generated, etc...  
--
-- If it's been 300 ms since the last debuff was added
-- consider all of them as being out.  While this isn't
-- a terribly appropriate delay, this is really a very
-- rare ocassion and mostly happens while wiping.
timer_frame:SetScript("OnUpdate", function()
	local now = GetTime()
	--print(now, last_pc_applied, now - last_pc_applied)
	if (now - last_pc_applied) >= .3 then
		calculate_my_pc_targets()
	end
end)

-- Table of functions to sort the group as requested
-- Note that the desc sorts are implemented as just tail calls of the
-- asc sorts with the arguments reversed.
local sort_functions = {}
sort_functions.name_asc = function(a, b)
	if not a then
		return false
	elseif not b then
		return true
	end

	local a_name, b_name = a[3], b[3]
	if not a_name then
		return false
	elseif not b_name then
		return true
	end

	return a_name < b_name
end

sort_functions.name_desc = function(a, b)
	return sort_functions.name_asc(b, a)
end

sort_functions.group_name_asc = function(a, b)
	if not a then
		return false
	elseif not b then
		return true
	end

	local a_group, b_group = a[4], b[4]
	if not a_group then
		return false
	elseif not b_group then
		return true
	end

	if a_group ~= b_group then
		return a_group < b_group
	end

	return sort_functions.name_asc(a, b)
end

sort_functions.group_name_desc = function(a, b)
	return sort_functions.group_name_asc(b, a)
end

function PitBull4_Anubarak:PARTY_MEMBERS_CHANGED()
	-- Fill the group_cache with the appropriate values and sort
	local count = GetNumRaidMembers()
	if count > 0 then
		-- In a raid
		group_cache[0] = delete(group_cache[0]) -- Delete the player entry that may exist from party	
		for i = 1, count do
			local entry = group_cache[i]
			if not entry then
				entry = new()
			end
			local _
			entry[1] = i
			entry[2] = UnitGUID("raid"..i)
			entry[3], _, entry[4] = GetRaidRosterInfo(i)
			group_cache[i] = entry
		end
	else
		count = GetNumPartyMembers()
		if count > 0 then
			-- In a party.  Add one to the count for the player
			count = count + 1
			for i = 1, count do
				local entry = group_cache[i]
				if not entry then
					entry = new()
				end
				local unit
				-- index 1 will be used as the player
				if i == 1 then
					unit = "player"
				else
					unit = "party" .. (i - 1)
				end
				entry[1] = i
				entry[2] = UnitGUID(unit)
				entry[3] = UnitName(unit)
				entry[4] = 1 -- party is always group 1 when not in raid
				group_cache[i] = entry
			end
		else
			-- not in a party, put the player in the cache
			local entry = group_cache[1]
			if not entry then
				entry = new()
			end
			entry[1] = 1
			entry[2] = UnitGUID("player")
			entry[3] = UnitName("player")
			entry[4] = 1
			group_cache[1] = entry
			count = 1
		end
	end

	-- Move any unused tables to the pool
	for i = count + 1,#group_cache do
		group_cache[i] = delete(group_cache[i])
	end

	sort(group_cache, sort_functions[self.db.profile.global.sort_method])
end

function PitBull4_Anubarak:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, log_event, src_guid, src_name, src_flags, dst_guid, dst_name, dst_flags, spell_id, spell_name, spell_school, ...)
	if log_event == "UNIT_DIED" then
		if queued_pc_targets[dst_guid] then
			queued_pc_targets[dst_guid] = nil
			queued_count = queued_count - 1
		end
		my_pc_targets[dst_guid] = nil
		self:UpdateForGUID(dst_guid)
	end

	if not (log_event == "SPELL_AURA_REMOVED" or log_event == "SPELL_AURA_REFRESH" or log_event == "SPELL_AURA_APPLIED") then
		return
	end
	local applications = penetrating_cold[spell_id]
	if not applications then return end

	-- Figure the size of the raid we're doing 10 or 25
--[[	if applications == 6 then
		raid_size = 25 -- special case for the WG test case.
	else ]]
		raid_size = applications * 5
--	end

	--print(GetTime(),log_event,raid_size)
	if log_event == "SPELL_AURA_REMOVED" then
		if queued_pc_targets[dst_guid] then
			queued_pc_targets[dst_guid] = nil
			queued_count = queued_count - 1
		end
		my_pc_targets[dst_guid] = nil
		self:UpdateForGUID(dst_guid)
	elseif log_event == "SPELL_AURA_REFRESH" or log_event == "SPELL_AURA_APPLIED" then
		if queued_count == 0 then
			-- next application so clear out the existing marks
			wipe(my_pc_targets)
			self:UpdateAll()
		end
		timer_frame:Show()
		queued_pc_targets[dst_guid] = true
		queued_count = queued_count + 1
		last_pc_applied = GetTime()
		if queued_count == applications then
			calculate_my_pc_targets()
		end
	end
end

function PitBull4_Anubarak:ZONE_CHANGED_NEW_AREA()
	-- Clear everything on a zone change (releasing for example)
	wipe(queued_pc_targets)
	wipe(my_pc_targtes)
	self:UpdateAll()
end

function PitBull4_Anubarak:GetTexture(frame)
	if my_pc_targets[frame.guid] then
		return texture
	else
		return nil
	end
end

function PitBull4_Anubarak:GetExampleTexture(frame)
	local unit = frame.unit or frame:GetName()

	local index = unit:match(".*(%d+)")
	if index then
		index = index+0
	else
		index = 0
	end
	if index == 0 or index % 5 == 0 then
		return texture
	end
end

PitBull4_Anubarak:SetGlobalOptionsFunction(function(self)
	return 'sort_method', {
		type = 'select',
		name = L['Sort method'],
		desc = L['Sort the group using this method.'],
		get = function(info)
			return PitBull4_Anubarak.db.profile.global.sort_method
		end,
		set = function(info, value)
			PitBull4_Anubarak.db.profile.global.sort_method = value
			sort(group_cache, sort_functions[value])
		end,
		values = {
			group_name_asc   = L["Group, name - ascending"],
			group_name_desc  = L["Group, name - descending"],
			name_asc         = L["Name, ascending"],
			name_desc        = L["Name, descending"],
		},
		width = 'double',
	}, 'announce', {
		type = 'toggle',
		name = L['Announce targets'],
		desc = L['Announce the targets to the raid.'],
		get = function(info)
			return PitBull4_Anubarak.db.profile.global.announce
		end,
		set = function(info, value)
			PitBull4_Anubarak.db.profile.global.announce = value
		end,
	}, 'assignments_10', {
		type = 'multiselect',
		dialogControl = "Dropdown",
		name = L['10-man assignments'],
		desc = L['Assignments used for 10-man raid instances.'],
		get = function(info, key)
			return PitBull4_Anubarak.db.profile.global.assignments[10][key]
		end,
		set = function(info, key, value)
			PitBull4_Anubarak.db.profile.global.assignments[10][key] = value
			PitBull4_Anubarak:UpdateAll()
		end,
		values = {
			[1] = "1",
			[2] = "2",
		},
	}, 'assignments_25', {
		type = 'multiselect',
		dialogControl = "Dropdown",
		name = L['25-man assignments'],
		desc = L['Assignments used for 25-man raid instances.'],
		get = function(info, key)
			return PitBull4_Anubarak.db.profile.global.assignments[25][key]
		end,
		set = function(info, key, value)
			PitBull4_Anubarak.db.profile.global.assignments[25][key] = value
			PitBull4_Anubarak:UpdateAll()
		end,
		values = {
			[1] = "1",
			[2] = "2",
			[3] = "3",
			[4] = "4",
			[5] = "5",
		},
	}
end)
